/* 
 * PDFlow -- chooser.h
 *
 *  Copyright (c) 2010 Alberto Cuda
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */
/*
 * This file is intended to address the common problem of choosing
 * the "most typical" value in a series. It is used e.g. to understand
 * the average space in a paragraph or the average weight of a page.
 * The idea here is to find the average value in a series that contains
 * also some samples that should be ignored because too small or too large.
 */

#ifndef chooser_h

#define chooser_h

#include "version.h"

struct ch_chooser {

	int size, idx;

	pdf_real_t *samples, *transf;
};

struct ch_outcome {

	pdf_real_t value, upper, lower;

	int lsamp, usamp;

};

enum s_status ch_init (struct ch_chooser *ch, int size);
void ch_add (struct ch_chooser *ch, pdf_real_t value);
void ch_choose (struct ch_chooser *ch, struct ch_outcome *cho);
void ch_draw_plots (struct ch_chooser *ch, struct ch_outcome *cho,
		    int base, const char *name);
void ch_destroy (struct ch_chooser *ch);


#endif
