/*
 * PDFlow -- layout.h
 *
 *  Copyright (c) 2010 Alberto Cuda
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */

#ifndef layout_h

#define layout_h

#include "chooser.h"
#include "grid/heur.h"
#include "grid/grid.h"
#include "grid/sig.h"

#define GR_L_MAX_LINES 5
#define GR_L_MAX_PROPS ARGS_MAX_BASELINES

struct gr_layout_segment {
	pdf_bool_t present;

	pdf_real_t bhalf, ehalf;

	struct gr_sig_signature sig;
};

struct gr_layout_line {

	pdf_real_t origin, gap;

	struct gr_line *line;

	struct gr_layout_segment begin, center, end;
};

enum gr_layout_occ_type {
	
	GR_L_OT_SUM,
	GR_L_OT_EVEN,
	GR_L_OT_ODD,

	GR_L_OT__LAST__

};

struct gr_layout_prop {

	struct gr_layout_prop *next;

	int lines, level;
	struct gr_layout_line line [GR_L_MAX_LINES + 2];

	int occurrences [GR_L_OT__LAST__];

	pdf_bool_t chosen;

	pdf_real_t min_weight, max_weight;
};

struct gr_layout_pout {
	int props;
	struct gr_layout_prop prop [GR_L_MAX_PROPS];
};

struct gr_layout_data {
	struct gr_layout_pout head, foot;

	struct cobj_rect clip;

	pdf_real_t min_weight, max_weight;
};

struct gr_layout {

	struct gr_layout_prop *head, *foot;

	struct gr_al_machine *dc;

	struct ch_chooser wch;

	struct ch_outcome cho;

	int samples;

	struct args_heur heur;

	pdf_bool_t hint_props;
};

enum s_status gr_layout_init (struct gr_layout *gly, struct args_heur *heur,
			      int samples);
enum s_status gr_layout_learn (struct gr_layout *gly, struct gspg_page *page,
			       pdf_real_t *weight, int pgno);
enum s_status gr_layout_add_hint (struct gr_layout *gly, 
				  struct gspg_page *page, 
				  int hlines, int flines);
void gr_layout_apply (struct gr_layout *gly, struct args_margins *amys,
		      struct gr_layout_data *ld);
		      
void gr_layout_destroy (struct gr_layout *gly);
void gr_layout_plot_prop (struct gr_layout *gly);
void gr_layout_plot_weight (struct gr_layout *gly, int base);


enum s_status gr_layout_setup_grid (struct gr_grid *grid,
				    struct cobj_rect *page_rect,
				    struct gr_layout_data *ld,
				    struct args_heur *heur);

#endif
