#ifndef pdfw_h

#define pdfw_h

#include "types/cobj.h"

#define PDFW_ARG_MAX 10

enum pdfw_opcode_type {

	PDFW_OP_SET_CTM,
	PDFW_OP_LINE_WIDTH,
	PDFW_OP_LINE_CAP,
	PDFW_OP_LINE_JOIN,
	PDFW_OP_MITER_LIMIT,
	PDFW_OP_DASH_PATTERN,
	PDFW_OP_RENDERING_INTENT,

	PDFW_OP_FONT,
	PDFW_OP_TEXT_MATRIX,
	PDFW_OP_CHAR_SPACE,
	PDFW_OP_WORD_SPACE,
	PDFW_OP_SCALE,
	PDFW_OP_LEADING,
	PDFW_OP_RENDER,
	PDFW_OP_RISE,

	PDFW_OP_GS,

	PDFW_OP_BT,
	PDFW_OP_ET,

	PDFW_OP_MOVE_TO,
	PDFW_OP_LINE_TO,
	PDFW_OP_BEZIER_TO,

	PDFW_OP_DRAW_TEXT,

	PDFW_OP_CLOSE_PATH,

	PDFW_OP_SHADING,
	PDFW_OP_XOBJECT,
	PDFW_OP_INLINE_IMAGE,

	PDFW_OP_SET_GRAY,
	PDFW_OP_SET_RGB,
	PDFW_OP_SET_CMYK,

	PDFW_OP_SET_CSPACE,
	PDFW_OP_SET_COLOUR,

	PDFW_OP_PUSH,
	PDFW_OP_POP,

	PDFW_OP_ADD_CLIPPING_PATH,

	/* -- Generated by the optimizer -- */
	PDFW_OP_CLOSE_SUBPATH,

	PDFW_OP_RECTANGLE
};

struct pdfw_writer;

struct pdfw_opcode {

	struct pdfw_opcode *next;

	enum pdfw_opcode_type type;

	struct cobj_matrix matrix;

	struct xref_obj_array array;

	int args;
	struct obj arg [PDFW_ARG_MAX];

	struct pdfw_opcode_desc *opd;
};

enum s_status pdfw_create (struct f_file *dest,
			   struct v_selection *sel,
			   struct pdfw_writer **pdfw);
void pdfw_destroy (struct pdfw_writer *pdfw);

void pdfw_destroy_opcode (struct pdfw_opcode *opc);

enum s_status pdfw_append (struct pdfw_writer *pdfw,
			   enum pdfw_opcode_type type, ...);

enum s_status pdfw_vcreate_opcode (struct pdfw_writer *pdfw,
				   enum pdfw_opcode_type type, va_list ap,
				   struct pdfw_opcode **opcp);

enum s_status pdfw_emit (struct pdfw_writer *pdfw,
			 struct pdfw_opcode *opc);

enum s_status pdfw_flush (struct pdfw_writer *pdfw);

#endif
