/*
 * PDFlow -- plot.c
 *
 *  Copyright (c) 2010 Alberto Cuda
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "status.h"
#include "plot.h"

#ifdef HAVE_PLPLOT
#include "plConfig.h"
#include "plplot.h"

static int inited;

enum s_status plp_init (void)
{
	return S_CSC_OK;
}

static void init (void)
{
	if (!inited) {
		plinit ();
		inited = 1;
	}
}

void plp_cleanup (void)
{
	if (inited)
		plend ();
}

enum s_status plp_histo (double label [], double data [], 
			 double thlo, double thhi,
			 int points, const char *title,
			 const char *xtitle, const char *ytitle)
{
	PLFLT x [4], y [4];
	double lo, hi, delta, xmin, xmax;
	int i;

	if (!points)
		return S_CSC_OK;

	init ();

	lo = hi = data [0];
	delta = label [points - 1] - label [0];

	for (i = 0; i < points; i++) {
		if (data [i] < lo)
			lo = data [i];
		if (data [i] > hi)
			hi = data [i];
		if (i && label [i] - label [i - 1] < delta)
			delta = label [i] - label [i - 1];
	}

	if (lo == hi) {
		if (lo > 0)
			lo = 0;
		else if (lo < 0)
			hi = 0;
		else
			hi = 1;
	}

	if (!delta)
		delta = 1;

	xmin = label [0] - delta / 2;
	xmax = label [points - 1] + delta / 2;

	plenv (xmin, xmax, lo, hi, 0, 0);

	for (i = 0; i < points; i++) {
		x [0] = x [1] = label [i] - delta / 2;
		x [2] = x [3] = label [i] + delta / 2;
		y [0] = y [3] = 0;
		y [1] = y [2] = data [i];
		if (thlo != thhi &&
		    (data [i] < thlo || data [i] > thhi))
			plcol0 (3);
		else
			plcol0 (1);
		plfill (4, x, y);
		pllsty (1);
		plcol0 (2);
		plline (4, x, y);
	}

	if (thlo != thhi) {
		plcol0 (4);
		pljoin (xmin, thlo, xmax, thlo);
		pljoin (xmin, thhi, xmax, thhi);
	}

	return S_CSC_OK;
}

#else

enum s_status plp_init (void)
{
	return S_CSC_OK;
}

void plp_cleanup (void)
{
	/* empty */
}

enum s_status plp_histo (double label [], double data [], 
			 double thlo, double thhi,
			 int points, const char *title,
			 const char *xtitle, const char *ytitle)
{
	return S_CSC_OK;
}

#endif


